import { wrapForCompatibility } from 'bpmn-js/lib/util/CompatibilityUtil'

export default function BpmnViewer (options) {
  this._options = options
}

BpmnViewer.prototype._randerBpmn = function (jsonData, bpmnModeler) {
  if (jsonData === undefined || jsonData.elements === undefined || jsonData.flows === undefined) {
    return
  }
  const modeling = bpmnModeler.get('modeling')
  // 实现步骤：1、找到页面里所有节点
  const elementRegistry = bpmnModeler.get('elementRegistry')
  this.nodeList = elementRegistry.filter(
    (item) => item.type === 'bpmn:UserTask' || item.type === 'bpmn:StartEvent' || item.type === 'bpmn:EndEvent' || item.type === 'bpmn:ExclusiveGateway' || item.type === 'bpmn:InclusiveGateway'
  )
  this.sequenceFlow = elementRegistry.filter(
    (item) => item.type === 'bpmn:SequenceFlow'
  )

  // 渲染节点颜色
  for (let i = 0; i < jsonData.elements.length; i++) {
    const v = jsonData.elements[i]
    if (v.current) {
      // 渲染当前节点
      const currentNode = this.nodeList.filter(node => node.id === v.id)[0]
      if (currentNode) {
        modeling.setColor(currentNode, this._color.current)
      }
    } else {
      if (v.completed) {
        // 渲染已经通过的节点
        const completedNode = this.nodeList.filter(node => node.id === v.id)[0]
        if (completedNode) {
          modeling.setColor(completedNode, this._color.passed)
        }
      }
    }
  }

  // 渲染线条颜色
  for (let i = 0; i < jsonData.flows.length; i++) {
    const v = jsonData.flows[i]
    if (v.completed) {
      const properties = v.properties; // 属性判断
      // 判断是否驳回
      let reject = false
      if (properties !== undefined && properties.length > 0) {
        for (let j = 0; j < properties.length; j++) {
          const property = properties[j]
          if (property.name === 'Documentation' && property.value !== '') {
            try {
              const lineValue = JSON.parse(property.value)
              if (lineValue.reject) {
                reject = true
              }
            } catch (e) {
              console.log(e)
            }
            break
          }
        }
      }
      const completedFlow = this.sequenceFlow.filter(flow => flow.id === v.id)[0]
      if (completedFlow) {
        if (reject) { // 驳回
          modeling.setColor(completedFlow, this._color.reject)
        } else { // 通过
          modeling.setColor(completedFlow, this._color.checked)
        }
      }
    }
  }
}

BpmnViewer.prototype._options = { jsonData: null, processId: null, processName: null, nodeClick: null, bpmnModeler: null }

BpmnViewer.prototype._color = {
  current: { stroke: '#030', fill: '#9f0' },
  passed: { stroke: '#009', fill: '#cff' },
  reject: { stroke: '#CD0000' },
  checked: { stroke: '#66CDAA' }
}

BpmnViewer.prototype.createNewDiagram = wrapForCompatibility(async function () {
  try {

    // 导入xml
    await this._options.bpmnModeler.importXML(this._createXml(this._options.processId, this._options.processName, this._options.jsonData))

    // 以下代码为：为节点注册鼠标悬浮事件
    const eventBus = this._options.bpmnModeler.get('eventBus')
    eventBus.on('element.click', (e) => {
      if (e.element.type === 'bpmn:UserTask') {
        const node = {}
        node.processId = this._options.processId
        node.processName = this._options.processName
        node.id = e.element.id
        node.bussiness = e.element.businessObject
        this._options.nodeClick(node)
      }
    })

    // 解析json
    this._randerBpmn(this._options.jsonData, this._options.bpmnModeler)
  } catch (err) {
    console.log(err.message, err.warnings)
  }
})

BpmnViewer.prototype.setColor = wrapForCompatibility(function (color) {
  this._color = color
})

BpmnViewer.prototype._createXml = function (processId, processName, jsonData) {
  // 解析json为xml
  let nodeList = '' // 节点
  let nodeShapeList = '' // 节点坐标
  if (jsonData.elements && jsonData.elements.length > 0) {
    jsonData.elements.forEach(nodeItem => {
      nodeItem.type = nodeItem.type.replace(nodeItem.type[0], nodeItem.type[0].toLowerCase())
      // 创建节点标签
      let nodeLabel = ''
      // 不同情况下创建不同元素属性
      if (nodeItem.type === 'userTask') {
        let userCode = ''
        let documentationLabel = ''
        if (nodeItem.properties && nodeItem.properties.length > 0) {
          nodeItem.properties.forEach(attr => {
            if (attr.name === 'Candidate groups') {
              userCode = attr.value && attr.value.length > 0 ? attr.value[0] : ''
            } else if (attr.name === 'Documentation') {
              documentationLabel = `<documentation>${attr.value}</documentation>`
            }
          })
        }
        nodeLabel = `<${nodeItem.type} id="${nodeItem.id}" name="${nodeItem.name ? nodeItem.name : ''}" flowable:formFieldValidation="${true}" flowable:candidateGroups="${userCode}">${documentationLabel}</${nodeItem.type}>`
      } else if (nodeItem.type === 'startEvent') {
        nodeLabel = `<${nodeItem.type} id="${nodeItem.id}" name="${nodeItem.name ? nodeItem.name : ''}" flowable:formFieldValidation="${true}"></${nodeItem.type}>`
      } else {
        nodeLabel = `<${nodeItem.type} id="${nodeItem.id}" name="${nodeItem.name ? nodeItem.name : ''}"></${nodeItem.type}>`
      }
      nodeList += nodeLabel
      // 创建节点位置标签
      const shapeLabel = `<bpmndi:BPMNShape bpmnElement="${nodeItem.id}" id="BPMNShape_${nodeItem.id}"><omgdc:Bounds height="${nodeItem.height}" width="${nodeItem.width}" x="${nodeItem.x}" y="${nodeItem.y}"></omgdc:Bounds></bpmndi:BPMNShape>`
      nodeShapeList += shapeLabel
    })
  }
  let flowsList = '' // 线
  let flowShapeList = '' // 线坐标
  if (jsonData.flows && jsonData.flows.length > 0) {
    jsonData.flows.forEach(flowItem => {
      flowItem.type = flowItem.type.replace(flowItem.type[0], flowItem.type[0].toLowerCase())
      // 创建线标签
      const flowLabel = `<${flowItem.type} id="${flowItem.id}" sourceRef="${flowItem.sourceRef}" targetRef="${flowItem.targetRef}" name="${flowItem.name ? flowItem.name : ''}"></${flowItem.type}>`
      flowsList += flowLabel
      // 创建线体坐标标签
      if (flowItem.waypoints && flowItem.waypoints.length > 0) {
        let flowWaypointLabel = ''
        flowItem.waypoints.forEach(pointItem => {
          flowWaypointLabel += `<omgdi:waypoint x="${pointItem.x}" y="${pointItem.y}"></omgdi:waypoint>`
        })
        const flowShapeLabel = `<bpmndi:BPMNEdge bpmnElement="${flowItem.id}" id="BPMNEdge_${flowItem.id}">${flowWaypointLabel}</bpmndi:BPMNEdge>`
        flowShapeList += flowShapeLabel
      }
    })
  }
  const xmlData = `
        <?xml version="1.0" encoding="UTF-8"?>
        <definitions xmlns="http://www.omg.org/spec/BPMN/20100524/MODEL" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:flowable="http://flowable.org/bpmn" xmlns:bpmndi="http://www.omg.org/spec/BPMN/20100524/DI" xmlns:omgdc="http://www.omg.org/spec/DD/20100524/DC" xmlns:omgdi="http://www.omg.org/spec/DD/20100524/DI" typeLanguage="http://www.w3.org/2001/XMLSchema" expressionLanguage="http://www.w3.org/1999/XPath" targetNamespace="http://www.flowable.org/processdef">
          <process id="${processId}" name="${processName}" isExecutable="true">
          <documentation>${processName}</documentation>
            ${nodeList}
            ${flowsList}
          </process>
           <bpmndi:BPMNDiagram id="BPMNDiagram_${processId}">
            <bpmndi:BPMNPlane bpmnElement="${processId}" id="BPMNPlane_${processId}">
              ${nodeShapeList}
              ${flowShapeList}
            </bpmndi:BPMNPlane>
          </bpmndi:BPMNDiagram>
        </definitions>
      `
  return xmlData
}
